=begin
#-----------------------------------------------------------------------
Copyright 2009 TIG
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided something the 
above copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------
Name        : Lines2Components.rb
Description : Turns Selected Lines into Instances of a Selected Component...
Usage       : Put this file into 'Plugins' folder.
              Open a Model.
              Place Components and Lines...
              Select an Instance of the required Component, and 
              at least one Line (Edge)***.
              Pick 'Lines2Components' from the 'Plugins' Menu.
              If the Selection doesn't contain Edges or an Instance 
              an error dialog tells you...
              You are prompted for the Scaling: 'Z' [default] or 'None'.
              If 'Z' is chosen then the Z-height (blue) of each Line*** 
              relative to the Z-height of the required Component sets 
              the 'Scaling' - for 3D 'angled' lines this is the end's
              Z-difference NOT the Line's length.
              One by one the Lines are turned into Instances of the 
              required Component, and Z-Scaled to suit.
              Z-Scaling is from the Instance's insertion point.
              If the Scaling is 'None' then the new inserted Instances 
              have the same Z-Scaling as the initially selected Instance.
              If the Scaling is 'None' then 'flat' Lines (equal end-Zs) 
              can be used, the insertion point is then the Line's 'start':
              otherwise each Instance's insertion point is taken to be 
              the lower of the Line's ends.  Note that any Scaling other 
              than in the Z for the Selected Instance is ignored in the 
              newly placed Instances...
Version     :	1.0 20090408	First Issue.
              1.1 20090408  XYZ Scaling removed and Selected Instance's
                            Z-Scaling used for Scaling=None.
#-----------------------------------------------------------------------
=end
require 'sketchup.rb'
#def
def lines2components()
  model=Sketchup.active_model
  model.start_operation("Lines2Components")
  entities=model.active_entities
  sel=model.selection
  sela=sel.to_a
  if not sela[1]
    UI.messagebox("Select Component Instance AND Line(s).\nExiting...")
    return nil
  end#if
  ins=nil
  lines=[]
  sela.each{|e|ins=e if e.typename=="ComponentInstance"
    lines.push(e) if e.typename=="Edge" }
  if not ins
    UI.messagebox("Selection must contain a Component Instance.\nExiting...")
    return nil
  end#if
  if not lines[0]
    UI.messagebox("Selection must contain Line(s).\nExiting...")
    return nil
  end#if
### Got instance and line(s) - get scaling options
  @scale="Z" if not @scale ### kept for this session as first picked ### v1.1
  lists=["Z|None"] ### v1.1
  prompts=["Scaling: "]
  values=[@scale]
  results=inputbox(prompts,values,lists,"Lines2Components")
  return nil if not results
  @scale=results[0]
  scaling=@scale
### find Z-height of ins and defn
  defn=ins.definition
  tran=ins.transformation ### v1.1
  insht=ins.bounds.max.z-ins.bounds.min.z ### v1.1
  defnht=defn.bounds.height ### v1.1
  insZscale=insht/defnht ### v1.1
### process lines
  zEquals=[]
  lines.each{|e|
    if e.start.position.z < e.end.position.z
      pt=e.start.position
      zMin=e.start.position.z
      zMax=e.end.position.z
      ht=zMax-zMin
    elsif e.end.position.z < e.start.position.z
      pt=e.end.position
      zMin=e.end.position.z
      zMax=e.start.position.z
      ht=zMax-zMin
    else# the same Z !
      pt=e.start.position
      zMin=e.start.position.z
      ht=nil
      zEquals.push(e)
    end#if
    if scaling=="None"
      tmpgrp=entities.add_group(e)
      tmpins=tmpgrp.to_component
      tmpdefn=tmpins.definition
      tmpins.definition=defn
      tmpdefn.entities.erase_entities(tmpdefn.entities.to_a)
      z=insZscale; x=1; y=1 ### v1.1
      t=tmpins.transformation ### v1.1
      tmpins.transformation=Geom::Transformation.scaling(pt,x,y,z)*t ### v1.1
      sel.add(tmpins)
    elsif scaling!="None" and ht
      tmpgrp=entities.add_group(e)
      tmpins=tmpgrp.to_component
      tmpdefn=tmpins.definition
      tmpins.definition=defn
      tmpdefn.entities.erase_entities(tmpdefn.entities.to_a)
      ### scale ht : defnht
      z=ht/defnht; x=1; y=1
      t=tmpins.transformation
      tmpins.transformation=Geom::Transformation.scaling(pt,x,y,z)*t
      sel.add(tmpins)
    end#if
  }#end lines.each
### check for flat lines
  if zEquals[1] and scaling!="None"
    UI.messagebox("The selection contained "+zEquals.length.to_s+" 'flat' Lines that weren't used.")
  elsif zEquals[0] and scaling!="None"
    UI.messagebox("The selection contained 1 'flat' Line that wasn't used.")
  end#if
###
  model.commit_operation
  return nil
end#def
###
#--------- menu -----------------------------
if(not file_loaded?("Lines2Components.rb"))
   UI.menu("Plugins").add_item("Lines2Components"){lines2components()}
end#if
file_loaded("Lines2Components.rb")
#-------------------------------------------